import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.base


class View(icemac.addressbook.browser.base.BaseView):
    """Base for views in icemac.ab.calendar."""

    __PACKAGE_ID__ = icemac.ab.calendar.interfaces.PACKAGE_ID
