from icemac.addressbook.i18n import _
import icemac.addressbook.browser.addressbook.startpage


calendar = icemac.addressbook.browser.addressbook.startpage.StartpageData(
    'icemac.ab.calendar.interfaces.ICalendar', 'index.html', _('Calendar'))
