import icemac.addressbook.browser.about.about


class CalendarIcon(icemac.addressbook.browser.about.about.IconProviderInfo):
    name = u'GraphicsFuel'
    url = u'http://www.graphicsfuel.com/2012/02/calendar-icon-psd'
