@media only print {
    #calendar table td:empty,
    #content .header,
    #content .body thead {
        display: none;
    }

    #content {
        border: 0px;
    }

    #timezone {
        float: right;
        font-size: small;
        position: absolute;
        right: 0px;
        bottom: 0px;
        margin-top: 0px;
    }


    #calendar table .day-0,
    #calendar table td.today {
        background-color: #fff;
    }

    #calendar .table h2 {
        display: block;
        font-size: 24px;
        position: absolute;
        right: 7px;
        top: 7px;
    }

    #calendar.year .table h2 {
        position: inherit;
        float: right;
    }

    #calendar.year .table .month {
        page-break-inside: avoid;
    }

    #calendar table td,
    #calendar table th {
        display: list-item;
        border: 0px;
        height: inherit;
        max-width: inherit;
        width: 50em;
        clear: left;
        list-style: outside none none;
    }

    #calendar table td .wrapper,
    #calendar table td dt,
    #calendar table td dd,
    #calendar table td dd a,
    #calendar table td dd ul,
    #calendar table td dd ul li,
    #calendar table td .weekday {
        float: left;
    }

    #calendar table td .number {
        text-align: right;
    }

    #calendar table td.day-0 .weekday {
        font-weight: bold;
    }

    #calendar table td .weekday {
        width: 25px;
    }

    #calendar table td .number {
        width: 20px;
        text-align: right;
    }

    #calendar table td dt {
        clear: left;
        margin-left: 5px;
    }

    #calendar table td dt:first-child {
        clear: none;
    }

    #calendar table td dl {
        float: left;
    }

    #calendar table td dl dd,
    div#content td ul {
        margin-left: 5px;
        padding-left: 0px;
    }
    #calendar table td dl dd ul.info li {
        margin-left: 20px;
    }

    #calendar table td span.number {
        float: left;
        color: #000;
        font-size: inherit;
        width: 1.5em;
    }

    #calendar table td dl dd span.info {
        display: inline;
        margin-left: 3px;
    }

}
