@media only screen {
    .no-screen {
        display: none;
    }
}

body.main {
    width: auto;
}

#content {
    background: #f6f6f6 none;
    border: 1px solid #999;
    padding: 5px;
    border-bottom-left-radius: 8px;
    border-bottom-right-radius: 8px;
    -moz-border-radius-bottomleft: 8px;
    -moz-border-radius-bottomright: 8px;
}

#timezone {
    float: right;
    clear: both;
}

#calendar .header {
    width: 100%;
}

#calendar .header .NW {
    background: url(img/bg-calendar.png) no-repeat;
    background-position: top left;
    width: 139px;
    height: 126px;
    float: left;
}


#calendar .header .NW-container {
    float: left;
    clear: both;
    margin-right: -148px;
    margin-top: 10px;
    width: 50%;
    height: 126px;
}

#calendar .header .NNW {
    background: url(img/bg-vertical.png) repeat;
    background-position: top;
    margin-right: 148px;
    margin-left: 74px;
    height: 126px;
    min-width: 115px;
}

#calendar .header .NNW .previous {
    float: right;
    margin-right: 5px;
}

#calendar .header a.flip {
    margin-top: 56px;
}

#calendar .header .center {
    background: url(img/bg-calendar.png) no-repeat;
    background-position: -108px 0px;
    width: 296px;
    height: 126px;
    float: left;
    margin-top: 10px;
}

#calendar .header .NO-container {
    float: right;
    margin-left: -148px;
    margin-top: 10px;
    width: 50%;
    height: 126px;
}

#calendar .header .NNO {
    background: url(img/bg-vertical.png) repeat;
    margin-left: 148px;
    margin-right: 74px;
    height: 126px;
    overflow: hidden;
}

#calendar .header .NNO .next {
    float: left;
    margin-left: 5px;
}

#calendar .header .NO {
    background: url(img/bg-calendar.png) no-repeat;
    background-position: top right;
    width: 139px;
    height: 126px;
    float: right;
}

#calendar .footer {
    clear: both;
}

#calendar .footer .SW {
    background: url(img/bg-calendar.png) no-repeat;
    background-position: bottom left;
    width: 75px;
    height: 113px;
    float: left;
}

#calendar .footer .SO {
    background: url(img/bg-calendar.png) no-repeat;
    background-position: bottom right;
    width: 75px;
    height: 113px;
    float: right;
}

#calendar .footer .S {
    background: url(img/bg-vertical.png) repeat;
    background-position: bottom;
    height: 113px;
    margin-right: 75px;
    margin-left: 75px;
}


#calendar .body {
    clear: both;
}

#calendar .body .W {
    width: 49px;
    min-height: 274px;
    background: url(img/bg-horizontal.png) repeat;
    float: left;
}

#calendar .body .O {
    width: 49px;
    min-height: 273px;
    background: url(img/bg-horizontal.png) repeat;
    background-position: right;
    float: right;
}

#calendar #calendar-select-form {
    padding-top: 55px;
}

#calendar #calendar-select-form .viewspace {
    text-align: center;
}

#calendar #calendar-select-form .row {
    display:inline-block;
}

#calendar #calendar-select-form div.widget {
    margin-left: 0;
}

#calendar #calendar-select-form .status,
#calendar #calendar-select-form .required-info,
#calendar #calendar-select-form .label {
    display: none;
}

#calendar #calendar-select-form .buttons {
    position: fixed;
    bottom: 0;
    left: 0;
    background-color: lightGrey;
    opacity: 0.85;
    padding: 0;
    width: 100%;
    border-radius: 0;
}

#calendar #calendar-select-form .buttons input {
    margin: 5px 0 5px 10px;
}

/* year tabular view */

#calendar.year h2 {
    padding-top: 10px;
}

#calendar.year h2:first-of-type {
    padding-top: 1px;
}

div#content #calendar table {
    background-color: white;
    border-spacing: 0;
    border-collapse: collapse;
}

div#content #calendar table th {
    border-top: 0;
}

div#content #calendar table th:first-child,
div#content #calendar table td:first-child {
    border-left: 0;
}

div#content #calendar table th:last-child,
div#content #calendar table td:last-child {
    border-right: 0;
}

#calendar table tr:last-child td {
    border-bottom: 0;
}

#calendar.year table {
    margin-top: 0;
}

#calendar table th,
#calendar table td {
    border: 1px solid #ccc;
    width: 1%;
    max-width: 1%;
}

#calendar table .day-0 {
    background-color: #F8E286;
}

#calendar table td.today {
    background-color: #F5CA85;
}


#calendar table th {
    font-weight: bold;
    text-align: center;
    padding: 2px;
}

#calendar table td {
    vertical-align: top;
    padding: 2px;
    height: 70px;
}

#calendar table td span.number {
    float: right;
    font-weight: bold;
}

#calendar table td .number {
    color: #aaa;
    font-size: 28px;
    margin-right: 2px;
}

#calendar table td a .number:hover {
    color: blue;
}

#calendar table td dl dd {
    margin-left: 15px;
}

#calendar table td dl dd.red {
    color: #FF0000;
}

#calendar table td dl dd.gray {
    color: #444444;
}

#calendar table td dl dd span.info {
    display: block;
    margin-left: 10px;
}

#calendar table td dl dd ul.info li {
    list-style-type: disc;
}


#calendar .menu ul {
    margin-top: -23px;
    text-align: center;
}

#calendar .menu li {
    display: inline-block;
    margin-left: 10px;
    margin-right: 10px;
}

#calendar .menu a {
    border: 1px solid #996633;
    border-radius: 8px;
    -moz-border-radius: 8px;
    padding: 0.1em 0.5em;
    text-decoration: none;
    color: #000000;
    background-color: #FFFFFF;
}

#calendar .menu a:hover,
#calendar .menu a.selected {
    background-color: #996633;
    color: #FFFFFF;
}

/* Style datetime Object widget inline so it looks like there is no object
   widget */
#content #form-widgets-datetime-row .label {
    display: none;
}
#content #form-widgets-datetime-row .widget .label {
    display: inherit;
}
#content #form-widgets-datetime-row .widget {
    margin-left: 0;
}
#content #form-widgets-datetime-row .widget .widget {
    margin-left: 1em;
}

/* Style the other persons widget */
#content #form-widgets-external_persons-row div.error div.error {
    display: none;
}
